<?php
namespace MascotCoreLovebite\Widgets\ImageGallery;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Utils;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Elementor Hello World
 *
 * Elementor widget for hello world.
 *
 * @since 1.0.0
 */
class TM_Elementor_Image_Gallery extends Widget_Base {
	public function __construct($data = [], $args = null) {
		parent::__construct($data, $args);
		$direction_suffix = is_rtl() ? '.rtl' : '';

		wp_enqueue_style( 'tm-image-gallery2-style', MASCOT_CORE_LOVEBITE_URL_PATH . 'assets/css/shortcodes/image-gallery' . $direction_suffix . '.css' );
	}

	/**
	 * Retrieve the widget name.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget name.
	 */
	public function get_name() {
		return 'tm-ele-image-gallery2';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget title.
	 */
	public function get_title() {
		return esc_html__( 'Image Gallery - Lovebite', 'mascot-core-lovebite' );
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget icon.
	 */
	public function get_icon() {
		return 'tm-elementor-widget-icon';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return array Widget categories.
	 */
	public function get_categories() {
		return [ 'tm' ];
	}

	/**
	 * Retrieve the list of scripts the widget depended on.
	 *
	 * Used to set scripts dependencies required to run the widget.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return array Widget scripts dependencies.
	 */
	public function get_script_depends() {
		return [ 'mascot-core-hellojs' ];
	}

	/**
	 * Skins
	 */
	protected function register_skins() {
		$this->add_skin( new Skins\Skin_Style_Current_Theme1( $this ) );
		$this->add_skin( new Skins\Skin_Style_Current_Theme2( $this ) );
		$this->add_skin( new Skins\Skin_Style_Current_Theme3( $this ) );
	}

	/**
	 * Register the widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function register_controls() {
		$this->start_controls_section(
			'tm_general',
			[
				'label' => esc_html__( 'General', 'mascot-core-lovebite' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		$this->add_control(
			'display_type',
			[
				'label' => esc_html__( "Display Type", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => [
					'grid'  =>  esc_html__( 'Grid', 'mascot-core-lovebite' ),
					'masonry' =>  esc_html__( 'Masonry', 'mascot-core-lovebite' ),
					'masonry-tiles' =>  esc_html__( 'Masonry Tiles', 'mascot-core-lovebite' ),
					'carousel'  =>  esc_html__( 'Carousel', 'mascot-core-lovebite' )
				],
				'default' => 'grid'
			]
		);
		$this->add_responsive_control(
			'img_min_height',
			[
				'label' => esc_html__( "Image Minimum Height", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'default' => [
					'unit' => 'px',
				],
				'range' => [
					'px' => [
						'min' => 100,
						'max' => 2000,
						'step' => 1,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .tm-image-gallery img' => 'min-height: {{SIZE}}{{UNIT}}; object-fit: cover;'
				],
				'condition' => [
					'display_type' => array('masonry')
				]
			]
		);
		$this->add_control(
			'columns',
			[
				'label' => esc_html__( "Columns Layout", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'description' => esc_html__( 'Define Columns Layout for Grid/Carousel', 'mascot-core-lovebite' ),
				'options' => [
					'1'  =>  '1',
					'2'  =>  '2',
					'3'  =>  '3',
					'4'  =>  '4',
					'5'  =>  '5',
					'6'  =>  '6',
					'7'  =>  '7',
					'8'  =>  '8',
					'9'  =>  '9',
					'10'  =>  '10',
				],
				'default' => '3'
			]
		);
		$this->add_control(
			'gutter',
			[
				'label' => esc_html__( "Gutter", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => mascot_core_lovebite_isotope_gutter_list_elementor(),
				'default' => 'gutter-15',
				'condition' => [
					'display_type' => array('grid')
				]
			]
		);
		$this->add_control(
			'featured_image_size', [
				'label' => esc_html__( "Featured Image Size", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => mascot_core_lovebite_get_available_image_sizes(),
			]
		);
		$this->end_controls_section();




		$this->start_controls_section(
			'general2',
			[
				'label' => esc_html__( 'Image Collection', 'mascot-core-lovebite' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		$repeater = new \Elementor\Repeater();
		//image
		$repeater->add_control(
			'logo', [
				'label' => esc_html__( "Logo", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::MEDIA,
				'default' => [
					'url' => Utils::get_placeholder_image_src(),
				],
			]
		);
		$repeater->add_control(
			'image_size', [
				'label' => esc_html__( "Image Size", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => mascot_core_lovebite_get_available_image_sizes(),
			]
		);
		$repeater->add_responsive_control(
			'img_min_height_individual',
			[
				'label' => esc_html__( "Image Minimum Height", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'default' => [
					'unit' => 'px',
				],
				'range' => [
					'px' => [
						'min' => 100,
						'max' => 2000,
						'step' => 1,
					],
				],
				'selectors' => [
					'{{WRAPPER}} {{CURRENT_ITEM}} .tm-image-gallery img' => 'min-height: {{SIZE}}{{UNIT}};'
				],
			]
		);
		$repeater->add_responsive_control(
			'img_max_height_individual',
			[
				'label' => esc_html__( "Or Maximum Height", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'default' => [
					'unit' => 'px',
				],
				'range' => [
					'px' => [
						'min' => 100,
						'max' => 2000,
						'step' => 1,
					],
				],
				'selectors' => [
					'{{WRAPPER}} {{CURRENT_ITEM}} .tm-image-gallery img' => 'width: 100%; max-height: {{SIZE}}{{UNIT}};'
				],
			]
		);
		$repeater->add_control(
			'title',
			[
				'label' => esc_html__( "Title", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::TEXT,
			]
		);
		$repeater->add_control(
			'title_tag',
			[
				'label' => esc_html__( "Title Tag", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => mascot_core_lovebite_heading_tag_list(),
				'default' => 'h5'
			]
		);
		$repeater->add_control(
			'subtitle',
			[
				'label' => esc_html__( "Sub Title", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::TEXT,
			]
		);
		$repeater->add_control(
			'subtitle_tag',
			[
				'label' => esc_html__( "Sub Title Tag", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => mascot_core_lovebite_heading_tag_list(),
				'default' => 'h4'
			]
		);
		$this->add_control(
			'gallery_images_array',
			[
				'label' => esc_html__( "Gallery Images", 'mascot-core-lovebite' ),
				'type' => \Elementor\Controls_Manager::REPEATER,
				'fields' => $repeater->get_controls(),
				'default' => [
					[
						'logo' => Utils::get_placeholder_image_src(),
					],
					[
						'logo' => Utils::get_placeholder_image_src(),
					],
					[
						'logo' => Utils::get_placeholder_image_src(),
					],
					[
						'logo' => Utils::get_placeholder_image_src(),
					],
					[
						'logo' => Utils::get_placeholder_image_src(),
					],
				],
			]
		);
		$this->end_controls_section();



		//Carousel Options
		$this->start_controls_section(
			'carousel_options', [
				'label' => esc_html__( 'Carousel Options', 'mascot-core-lovebite' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
				'condition' => [
					'display_type' => array('carousel')
				]
			]
		);
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 1, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 2, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 3, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 4, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 5, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 6, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 7, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 8, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 9, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 10, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 11, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 12, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 13, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_arraylist( $this, 14, '', array('display_type' => array('carousel') ) );
		$this->end_controls_section();

		//Carousel Arrow Navigation Options
		$this->start_controls_section(
			'carousel_arrow_nav_options', [
				'label' => esc_html__( 'Carousel Arrow Navigation Options', 'mascot-core-lovebite' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
				'condition' => [
					'display_type' => array('carousel')
				]
			]
		);
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 1, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 2, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 3, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 4, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 5, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 6, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 7, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 8, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 9, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 10, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 11, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 12, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 13, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 14, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 15, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 16, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 17, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 18, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 19, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 20, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 21, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 22, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 23, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 24, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 25, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 26, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 27, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 28, '');
		mascot_core_lovebite_get_owl_carousel_nav_arraylist( $this, 29, '');
		$this->end_controls_section();






		//Carousel Arrow Navigation Options
		$this->start_controls_section(
			'carousel_arrow_nav_bs5_breakpoints_options', [
				'label' => esc_html__( 'Carousel Arrow Nav Bootstrap5 Breakpoints', 'mascot-core-lovebite' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
				'condition' => [
					'display_type' => array('carousel')
				]
			]
		);
		mascot_core_lovebite_get_owl_carousel_nav_bs5_breakpoints_arraylist( $this, 1, '');
		mascot_core_lovebite_get_owl_carousel_nav_bs5_breakpoints_arraylist( $this, 2, '');
		mascot_core_lovebite_get_owl_carousel_nav_bs5_breakpoints_arraylist( $this, 3, '');
		mascot_core_lovebite_get_owl_carousel_nav_bs5_breakpoints_arraylist( $this, 4, '');
		mascot_core_lovebite_get_owl_carousel_nav_bs5_breakpoints_arraylist( $this, 5, '');

		mascot_core_lovebite_get_owl_carousel_nav_bs5_breakpoints_arraylist( $this, 6, '');
		mascot_core_lovebite_get_owl_carousel_nav_bs5_breakpoints_arraylist( $this, 7, '');
		mascot_core_lovebite_get_owl_carousel_nav_bs5_breakpoints_arraylist( $this, 8, '');
		mascot_core_lovebite_get_owl_carousel_nav_bs5_breakpoints_arraylist( $this, 9, '');
		mascot_core_lovebite_get_owl_carousel_nav_bs5_breakpoints_arraylist( $this, 10, '');
		$this->end_controls_section();






		//Carousel Bullets/Dots Options
		$this->start_controls_section(
			'carousel_arrow_dots_options', [
				'label' => esc_html__( 'Carousel Bullets/Dots Options', 'mascot-core-lovebite' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
				'condition' => [
					'display_type' => array('carousel')
				]
			]
		);
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 1, '', array('display_type' => array('carousel') ) );
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 2, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 3, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 4, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 5, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 6, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 7, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 8, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 9, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 10, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 11, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 12, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 13, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 14, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 15, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 16, '');
		mascot_core_lovebite_get_owl_carousel_dots_arraylist( $this, 17, '');
		$this->end_controls_section();
	}

	/**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function render() {
		$settings = $this->get_settings_for_display();
		mascot_core_lovebite_wp_enqueue_script_lightgallery();

		//classes
		$classes = array();
		if( $settings['display_type'] == 'grid' ) {
			$classes[] = 'grid-' . $settings['columns'];
		}
		$settings['classes'] = $classes;
		
		$settings['holder_id'] = mascot_core_lovebite_get_isotope_holder_ID('gallery');

		//Owl Carousel Data
		$settings['owl_carousel_data_info'] = mascot_core_lovebite_prepare_owlcarousel_data_from_params( $settings );

		$settings['settings'] = $settings;

		//Produce HTML version by using the parameters (filename, variation, folder name, parameters, shortcode_ob_start)
		$html = mascot_core_lovebite_get_shortcode_template_part( 'gallery', $settings['display_type'], 'image-gallery/tpl', $settings, true );

		echo $html;
	}
}
